from ...._utils import ANSICodeBase

BOLD = ANSICodeBase(1)
UNDERLINE = ANSICodeBase(4)
STRIKETHROUGH = ANSICodeBase(9)
SWAPCOLOR = ANSICodeBase(7)
BLACK = ANSICodeBase(30)
RED = ANSICodeBase(31)
GREEN = ANSICodeBase(32)
YELLOW = ANSICodeBase(33)
BLUE = ANSICodeBase(34)
MAGENTA = ANSICodeBase(35)
CYAN = ANSICodeBase(36)
WHITE = ANSICodeBase(37)
DEFAULT = ANSICodeBase(39)
LBLACK = ANSICodeBase(90)
LRED = ANSICodeBase(91)
LGREEN = ANSICodeBase(92)
LYELLOW = ANSICodeBase(93)
LBLUE = ANSICodeBase(94)
LMAGENTA = ANSICodeBase(95)
LCYAN = ANSICodeBase(96)
LWHITE = ANSICodeBase(97)

RESET = ANSICodeBase(0)
FGRESET = ANSICodeBase(39)