# -*- coding: utf-8 -*-

"""
Utility for use by package-internal modules
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Contains functions, classes etc. for use by the package-internal modules.
"""

from ._ansi_code_base import ANSICodeBase