# -*- coding: utf-8 -*-

"""
Foreground
~~~~~~~~~~
ANSI codes to change the foreground (text) color
"""

from ._ansii_codes import (
    BLACK, BLUE, BOLD, CYAN, DEFAULT, FGRESET, GREEN, LBLACK, LBLUE, LCYAN, LGREEN, LMAGENTA, LRED,
    LWHITE, LYELLOW, MAGENTA, RED, RESET, STRIKETHROUGH, SWAPCOLOR, UNDERLINE, WHITE, YELLOW
)