# -*- coding: utf-8 -*-

"""
Background
~~~~~~~~~~
ANSI codes to change the background color
"""

from ._ansii_codes import (
    BGRESET, BLACK, BLUE, CYAN, DEFAULT, GREEN, LBLACK, LBLUE, LCYAN, LGREEN, LMAGENTA, LRED,
    LWHITE, LYELLOW, MAGENTA, RED, RESET, SWAPCOLOR, WHITE, YELLOW
)