from typing import overload, Self


class ANSICodeBase:
    @overload
    def __init__(self, *values: int): ...
    @overload
    def __init__(self, values: str): ...
    def __repr__(self) -> str: ...
    def __add__(self, other: str | Self) -> str | Self: ...
    def __radd__(self, other: str | Self) -> str | Self: ...
    def __call__(self) -> None:

    value: str
    ansi_numbers: list[int]